package com.company.bassdkot.user

import com.company.bassdkot.entity.Civility
import com.company.bassdkot.entity.User
import com.company.bassdkot.test_support.AuthenticatedAsAdmin
import io.jmix.core.DataManager
import io.jmix.core.security.UserRepository
import io.jmix.data.PersistenceHints
import jakarta.persistence.EntityManager
import jakarta.persistence.PersistenceContext
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.transaction.support.TransactionTemplate


/**
 * Sample integration test for the User entity.
 */
@SpringBootTest
@ExtendWith(AuthenticatedAsAdmin::class)
class CommonTest {

    @Autowired
    lateinit var dataManager: DataManager

    @Autowired
    lateinit var passwordEncoder: PasswordEncoder

    @Autowired
    lateinit var userRepository: UserRepository

    @Autowired
    lateinit var transactionTemplate: TransactionTemplate

    private var savedUser: User? = null

    @PersistenceContext
    lateinit var em: EntityManager



    @Test
    fun test_saveAndLoad() {
        // Create and save a new User
        val user = dataManager.create(User::class.java)
        user.username = "test-user-" + System.currentTimeMillis()
        user.password = passwordEncoder.encode("test-passwd")
        savedUser = dataManager.save(user)

        // Check the new user can be loaded
        val loadedUser = dataManager.load(User::class.java).id(user.id!!).one()
        assertThat(loadedUser).isEqualTo(user)

        // Check the new user is available through UserRepository
        val userDetails = userRepository.loadUserByUsername(user.username)
        assertThat(userDetails).isEqualTo(user)
    }

    @Test
    fun testSoftDeletion() {
        var initialCivility = dataManager.create(Civility::class.java)
        initialCivility.code = "code"
        initialCivility = dataManager.save(initialCivility)

        dataManager.remove(initialCivility)

        val testEntities = dataManager.load(Civility::class.java).all().list()
        assertThat(testEntities).isEmpty()//entity soft deleted, not loaded with enabled soft-deletion

        val testEntitiesWithoutSD = dataManager.load(Civility::class.java)
                .all()
                .hint(PersistenceHints.SOFT_DELETION, false)
                .list()

        assertThat(testEntitiesWithoutSD).isNotEmpty()
        assertThat(testEntitiesWithoutSD.size).isEqualTo(1)
        assertThat(testEntitiesWithoutSD.get(0).code).isEqualTo(initialCivility.code)
    }

    @BeforeEach
    fun cleanData() {
        cleanTables()
    }

    @AfterEach
    fun tearDown() {
        if (savedUser != null)
            dataManager.remove(savedUser)
        cleanTables()
    }

    fun cleanTables() {
        transactionTemplate?.executeWithoutResult {
            em.createNativeQuery("delete from TEST_CIVILITY").executeUpdate()
        }
    }
}
